-----------------------------------------------------------------
--  lcd124.vhd
--			Interface-IC mit IO-Ring
--			========================
--
--  07.10.97	Bodo Wenzel	Erstellung
--  26.03.98	Bodo Wenzel	Hardware fertig
--  09.04.98	Bodo Wenzel	Test
--  26.05.98	Bodo Wenzel	Jumper fuer Bildlage
-----------------------------------------------------------------

-- Interface ----------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity lcd124 is
  port (clock:    in    std_ulogic;
        vsync_jp: inout std_logic_vector(4 downto 0);
        hfreq_jp: inout std_logic_vector(1 downto 0);
        hsync_jp: inout std_logic_vector(1 downto 0);
        pixel_jp: inout std_logic_vector(2 downto 0);
        vsync:    in    std_ulogic;
        hsync:    in    std_ulogic;
        mono:     in    std_ulogic;
        pll:      out   std_ulogic;
        rom_ram:  out   std_ulogic;
        n_oe:     out   std_ulogic;
        n_we:     out   std_ulogic;
        addr:     out   std_ulogic_vector(14 downto 0);
        data:     inout std_logic_vector(7 downto 0);
        s:        out   std_ulogic;
        cp1:      out   std_ulogic;
        cp2:      out   std_ulogic;
        du:       out   std_ulogic_vector(3 downto 0);
        dl:       out   std_ulogic_vector(3 downto 0));
end;

-- Implementation -----------------------------------------------

architecture beh of lcd124 is
  component top
  port (clock_p:  in  std_ulogic;
        ctl_byte: out std_ulogic;
        clock_b:  in  std_ulogic;
        vsync_jp: in  std_ulogic_vector(4 downto 0);
        hfreq_jp: in  std_ulogic_vector(1 downto 0);
        hsync_jp: in  std_ulogic_vector(1 downto 0);
        pixel_jp: in  std_ulogic_vector(2 downto 0);
        vsync:    in  std_ulogic;
        hsync:    in  std_ulogic;
        mono:     in  std_ulogic;
        pll_up:   out std_ulogic;
        pll_dn:   out std_ulogic;
        rom_ram:  out std_ulogic;
        n_oe:     out std_ulogic;
        n_we:     out std_ulogic;
        rd_addr:  out std_ulogic_vector(14 downto 0);
        rd_data:  in  std_ulogic_vector(7 downto 0);
        wr_addr:  out std_ulogic_vector(14 downto 0);
        wr_data:  out std_ulogic_vector(7 downto 0);
        s:        out std_ulogic;
        cp1:      out std_ulogic;
        cp2:      out std_ulogic;
        du:       out std_ulogic_vector(3 downto 0);
        dl:       out std_ulogic_vector(3 downto 0));
  end component;

  component gclk
  port (i: in  std_ulogic;
        o: out std_ulogic);
  end component;

  component aclk
  port (i: in  std_ulogic;
        o: out std_ulogic);
  end component;

  component pullup
  port (o: out std_logic);
  end component;

  component ibuf
  port (i: in  std_logic;
        o: out std_ulogic);
  end component;

  signal clock_p:  std_ulogic;
  signal ctl_byte: std_ulogic;
  signal clock_b:  std_ulogic;

  signal i_vsync_jp: std_ulogic_vector(4 downto 0);
  signal i_hfreq_jp: std_ulogic_vector(1 downto 0);
  signal i_hsync_jp: std_ulogic_vector(1 downto 0);
  signal i_pixel_jp: std_ulogic_vector(2 downto 0);

  signal pll_up: std_ulogic;
  signal pll_dn: std_ulogic;

  signal rd_addr: std_ulogic_vector(14 downto 0);
  signal rd_data: std_ulogic_vector(7 downto 0);
  signal wr_addr: std_ulogic_vector(14 downto 0);
  signal wr_data: std_ulogic_vector(7 downto 0);
begin
  c_top: top port map
    (clock_p,ctl_byte,clock_b,
     i_vsync_jp,i_hfreq_jp,i_hsync_jp,i_pixel_jp,
     vsync,hsync,mono,
     pll_up,pll_dn,
     rom_ram,n_oe,n_we,
     rd_addr,rd_data,wr_addr,wr_data,
     s,cp1,cp2,du,dl);

  c_clk1: aclk port map (i=>clock,o=>clock_p);

  c_clk2: gclk port map (i=>ctl_byte,o=>clock_b);

  g_vsync_jp:
  for i in vsync_jp'range generate
    r_vsync_jp: pullup port map (o=>vsync_jp(i));
    p_vsync_jp: ibuf port map (i=>vsync_jp(i),o=>i_vsync_jp(i));
  end generate;

  g_hfreq_jp:
  for i in hfreq_jp'range generate
    r_hfreq_jp: pullup port map (o=>hfreq_jp(i));
    p_hfreq_jp: ibuf port map (i=>hfreq_jp(i),o=>i_hfreq_jp(i));
  end generate;

  g_hsync_jp:
  for i in hsync_jp'range generate
    r_hsync_jp: pullup port map (o=>hsync_jp(i));
    p_hsync_jp: ibuf port map (i=>hsync_jp(i),o=>i_hsync_jp(i));
  end generate;

  g_pixel_jp:
  for i in pixel_jp'range generate
    r_pixel_jp: pullup port map (o=>pixel_jp(i));
    p_pixel_jp: ibuf port map (i=>pixel_jp(i),o=>i_pixel_jp(i));
  end generate;

  pll <= '1' when pll_up='1' and pll_dn='0'
    else '0' when pll_up='0' and pll_dn='1'
    else 'Z';

  addr(14 downto 0) <= rd_addr when ctl_byte='1'
                  else wr_addr;

  data <= (others => 'Z') when ctl_byte='1'
     else std_logic_vector(wr_data);
  rd_data <= std_ulogic_vector(data);
end;

-- Ende ---------------------------------------------------------
