-----------------------------------------------------------------
--  top.vhd
--			SM124-Emulator fuer Monochrome-LCD
--			==================================
--
--  26.09.97	Bodo Wenzel	PLL
--  03.10.97	Bodo Wenzel	LCD-Interface
--  07.10.97	Bodo Wenzel	Video-Interface
--  27.03.98	Bodo Wenzel	Hardware fertig
--  09.04.98	Bodo Wenzel	Test
--  26.05.98	Bodo Wenzel	Jumper fuer Bildlage
-----------------------------------------------------------------

-- Interface ----------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity top is
  port (clock_p:  in  std_ulogic;
        ctl_byte: out std_ulogic;
        clock_b:  in  std_ulogic;
        vsync_jp: in  std_ulogic_vector(4 downto 0);
        hfreq_jp: in  std_ulogic_vector(1 downto 0);
        hsync_jp: in  std_ulogic_vector(1 downto 0);
        pixel_jp: in  std_ulogic_vector(2 downto 0);
        vsync:    in  std_ulogic;
        hsync:    in  std_ulogic;
        mono:     in  std_ulogic;
        pll_up:   out std_ulogic;
        pll_dn:   out std_ulogic;
        rom_ram:  out std_ulogic;
        n_oe:     out std_ulogic;
        n_we:     out std_ulogic;
        rd_addr:  out std_ulogic_vector(14 downto 0);
        rd_data:  in  std_ulogic_vector(7 downto 0);
        wr_addr:  out std_ulogic_vector(14 downto 0);
        wr_data:  out std_ulogic_vector(7 downto 0);
        s:        out std_ulogic;
        cp1:      out std_ulogic;
        cp2:      out std_ulogic;
        du:       out std_ulogic_vector(3 downto 0);
        dl:       out std_ulogic_vector(3 downto 0));
end;

-- Implementation -----------------------------------------------

architecture beh of top is
  component control
  port (clock:    in  std_ulogic;
        ctl_byte: out std_ulogic;
        ctl_wr:   out std_ulogic);
  end component;

  component pll
  port (clock:   in  std_ulogic;
        hsync_x: in  std_ulogic;
        hsync_i: in  std_ulogic;
        pll_up:  out std_ulogic;
        pll_dn:  out std_ulogic);
  end component;

  component video
  port (clock_p:  in  std_ulogic;
        clock_b:  in  std_ulogic;
        vsync_jp: in  std_ulogic_vector(4 downto 0);
        hfreq_jp: in  std_ulogic_vector(1 downto 0);
        hsync_jp: in  std_ulogic_vector(1 downto 0);
        pixel_jp: in  std_ulogic_vector(2 downto 0);
        vsync_x:  in  std_ulogic;
        hsync_x:  in  std_ulogic;
        hsync_i:  out std_ulogic;
        mono:     in  std_ulogic;
        ctl_wr:   in  std_ulogic;
        addr:     out std_ulogic_vector(14 downto 0);
        data:     out std_ulogic_vector(7 downto 0);
        n_we:     out std_ulogic;
        catching: out std_ulogic);
  end component;

  component lcd
  port (clock: in  std_ulogic;
        addr:  out std_ulogic_vector(14 downto 0);
        data:  in  std_ulogic_vector(7 downto 0);
        s:     out std_ulogic;
        cp1:   out std_ulogic;
        cp2:   out std_ulogic;
        du:    out std_ulogic_vector(3 downto 0);
        dl:    out std_ulogic_vector(3 downto 0));
  end component;

  signal i_ctl_byte: std_ulogic;
  signal ctl_wr:     std_ulogic;

  signal hsync_i: std_ulogic;

  signal catching: std_ulogic;
begin
  c_control: control port map
    (clock_p,i_ctl_byte,ctl_wr);

  c_pll: pll port map
    (clock_p,
     hsync,hsync_i,
     pll_up,pll_dn);

  c_video: video port map
    (clock_p,clock_b,
     vsync_jp,hfreq_jp,hsync_jp,pixel_jp,
     vsync,hsync,hsync_i,mono,
     ctl_wr,wr_addr,wr_data,n_we,
     rom_ram);

  c_lcd: lcd port map
    (clock_b,
     rd_addr,rd_data,
     s,cp1,cp2,du,dl);

  ctl_byte <=  i_ctl_byte;

  n_oe  <= not i_ctl_byte;
end;

-- Ende ---------------------------------------------------------
